#ifndef polygons_H
#define polygons_H

/**
 ** A point is described as an array of 2 double.
 **
 ** A polygon is described by :
 **
 **     - an int 'nbPts' that indicates the number of points.
 **
 **     - an array 'poly' of '2*nbPts' double with :
 **
 **                poly[0]    poly[1]     : 1-st point coord
 **                poly[2]    poly[3]     : 2-nd point coord
 **                ...
 **                poly[2*i]  poly[2*i+1] : i-th point coord
 **
 **   Polygons can be closed or open (ie, 1st and last points can 
 **   be identical).
 **
 **   Their points can be given in trigo or reverse order
 **   (does not depend on matter side)
 **
 **/

/*----------------------------------------------------------------------
 *  PLpolygonInterPolygon   - 
 *
 * Description : Test polygon/polygon surfaces intersection
 * Input       : 2 polygons
 * Returns     : TRUE if intersection (including the border), FALSE otherwise
 */
extern int PLpolygonInterPolygon (int nbPts1, double *poly1, 
				  int nbPts2, double *poly2);

/*----------------------------------------------------------------------
 * PLrectangleInterPolygon   - 
 *
 * Description : Test if a rectangle intersect a polygon
 * Input       : 1 polygon with 4 vertices and 1 general polygon
 * Returns     : TRUE if intersection (including the border), FALSE otherwise
 *               WARNING: FALSE if the polygon is entirely inside the rectangle
 */
extern int PLrectangleInterPolygon (double rectangle[2*4], 
				    int nbPts, double *poly);
#define PLrobotInterPolygon PLrectangleInterPolygon

/*----------------------------------------------------------------------
 * PLpolygonInsidePolygon   - 
 *
 * Description : Test if the polygon poly1 is inside the polygon poly2
 * Input       : 2 polygons (at least 1 point for 1st one and 3 for the 2nd)
 * Returns     : TRUE if entirely inside, FALSE otherwise
 */
extern int PLpolygonInsidePolygon (int nbPts1, double *poly1, 
				   int nbPts2, double *poly2);

/*----------------------------------------------------------------------
 * PLrectangleInsidePolygon   - 
 *
 * Description : Test if a rectangle is entirely inside a polygon
 * Input       : 1 polygon with 4 vertices and 1 general polygon
 * Returns     : TRUE if entirely inside, FALSE otherwise
 */
extern int PLrectangleInsidePolygon (double rectangle[2*4], 
				     int nbPts, double *poly);
#define PLrobotInsidePolygon PLrectangleInsidePolygon

/*----------------------------------------------------------------------
 * PLpointInPolygon   - 
 *
 * Description : Test if a point is inside a polygon
 * Input       : 1 point and 1 general polygon
 * Returns     : TRUE if inside or on the edge, FALSE otherwise
 */
extern int PLpointInPolygon (double point[2], int nbPts, double *poly);


/*----------------------------------------------------------------------
 * PLpointSegDist
 *
 * Description : mputes dist between a point and a segment
 * Input       : 1 point and 1 segment
 * Returns     : distance
 */

double PLpointSegDist (double point[2], double *seg);

/*----------------------------------------------------------------------
 * PLprojPointOnSeg
 *
 * Description : Computes projection of a pt on a line defined by a seg.
 * Input       : 1 point, 1 segment and wanted dist between seg and 
 *               "proj pt" (0 for a point on the seg)
 * Returns     : 0 if proj on seg, 1 if outside ext1, 2 if outside ext2
 *               and the point proj on the line.
 */

int PLprojPointOnSeg (double pt[2], double *seg, 
		      double requiredDistToSeg, double *proj);

/*----------------------------------------------------------------------
 * PLpointNearestPolygonEdge
 *
 * Description : Look for nearest edge of a polygon from a given point
 * Input       : 1 point and 1 general polygon
 * Returns     : index i of nearest edge [Pi, Pi+1]
 */

extern int PLpointNearestPolygonEdge (double point[2], int nbPts, double *poly,
				      double *minDist);

/*----------------------------------------------------------------------
 * PLtestIntersectSegments   - 
 *
 * Description : Test segments intersection
 * Input       : 4 points: 2 for the 1st segment and 2 for the 2nd one
 * Returns     : TRUE if the intersect, FALSE otherwise
 */
extern int PLtestIntersectSegments (double p1s1[2], double p2s1[2],
				     double p1s2[2], double p2s2[2]);
/*----------------------------------------------------------------------
 * PLintersectSegments   - 
 *
 * Description : Computes segments intersection
 * Input       : 4 points: 2 for the 1st segment and 2 for the 2nd one
 * Output      : 1 point: the intersection if any
 * Returns     : TRUE if the intersect, FALSE othewise
 */
int PLintersectSegments (double p1s1[2], double p2s1[2],
			 double p1s2[2], double p2s2[2],
			 double inter[2]);

/*----------------------------------------------------------------------
 * PLtestClosedPolygon   - 
 *
 * Description : Test if a polygon is closed, 
 *               ie, if 1st and last points are identical
 * Input       : 1 polygon
 * Returns     : TRUE if it is closed, FALSE otherwise
 */
extern int PLtestClosedPolygon(int nbPts, double *poly);


/*----------------------------------------------------------------------
 * PLbissectorPt
 *
 * Description : computes a point on a bissector at a given distance
 *               from the vertice and/or from the segs
 * Input       : 3 points (p1 is the vertice) and 1 required distance
 * Outputs     : the point at dist from vertice (if non NULL) and
 *             : the point at dist from segs (if non NULL) 
 * Returns     : TRUE
 */

extern int PLbissectorPt(double *p0, double *p1, double *p2,
			 double dist, double *fromVertice, 
			 double *fromSeg);

/*----------------------------------------------------------------------
 * PLpolygonMinMax   - 
 *
 * Description : Computes min, max polygon coordinates
 * Input       : 1 polygon 
 * Output      : the 4-th coordinates
 * Returns     : void
 */
extern void PLpolygonMinMax(int nbPts, double *poly,
			    double *xMin, double *yMin, 
			    double *xMax, double *yMax);


/*----------------------------------------------------------------------
 * PLgrowSeg
 *
 * Description : Computes a seg parallel to a gievn seg at a given signed
 *               distance. Usefull to grow a segment to make a rectangle.
 *               For a positive input distance, the new seg
 *               will on the left for someone walking from pt1 to pt2.
 * Input       : 1 input seg, the distance, 1 output seg
 * Returns     : TRUE 
 */

int PLgrowSeg(double *src, double dist, double *dst);

/*----------------------------------------------------------------------
 * PLgrowPolygon
 *
 * Description : Make a polygon bigger (or smaller if dist < 0)
 *               For a positive input distance, the new edges
 *               will on the left for someone walking from pt1 to pt2.
 *               In other word, a clock wise polygon will be enlarged
 *               with dist > 0.
 *               polygon must be given in clock wise.
 * Input       : input poly, dist, output poly (same size)
 * Returns     : TRUE 
 */
extern int PLgrowPolygon(int nbPts, double *src, double dist, double *dst);

/*----------------------------------------------------------------------
 * PLprintPolygon
 *
 * Description : Print a polygon
 * Input       : 1 filedescripor and 1 polygon
 * Returns     : TRUE if it is closed, FALSE otherwise
 */
void PLprintPolygon(FILE *pfile, int nbPts, double *poly, int fullDataFlag);
/*----------------------------------------------------------------------
 * PLscanOneH2Polygon
 *
 * Description : Scan polygon of type "H2" one by one
 *               H2 obstacles file format :
 *                 OBSTACLES  [any comments...]
 *                 x0  y0
 *                 x1  y1
 *                 [...]
 *                 xi  yi
 *                 END     [any comments...]
 *                 [...]
 *                 x0  y0
 *                 x1  y1
 *                 [...]
 *                 xj  yj
 *                 END    ([ny comments...]
 *
 *               If scan is OK the polygon is allocated by this function
 *
 * Input       : 1 file descriptor 
 *               1 reference on the number of points of the read polygon
 * Returns     : reference on the polygon or NULL
 */
double * PLscanOneH2Polygon(FILE *pfile, int *nbPts);

/*----------------------------------------------------------------------
 * PLpointInHorizontalRectangle   - 
 *
 * Description : Test if a point is inside an horizontal rectangle
 * Input       : 1 point and 1 rectangle xmin, ymin, xmax, ymax
 * Returns     : TRUE if inside or on the edge, FALSE otherwise
 */

int PLpointInHorizontalRectangle (double point[2], double rect[2*2]);

/*----------------------------------------------------------------------
 * PLclipSegment   - 
 *
 * Description : Clip seg according to rectangle
 * Input       : Extremities ptSeg1 and ptSeg2 of the segment
 *               1 rectangle xmin, ymin, xmax, ymax
 *               Extremities pt1out and pt2out of clipped segment
 * Returns     : 0 if the segment is outside the rectangle, 
 *               1 if the segment is inside the rectangle, 
 *               2 if the segment has been clipped
 */
int PLclipSegment(double ptSeg1[2], double ptSeg2[2], 
		  double rect[2*2],
		  double pt1out[2], double pt2out[2]);

#endif /* POLYGONS_H */
